<?php
/**
 * @file
 * Admin page callbacks for recipient configuration.
 */

/**
 * Form function for selecting available recipient lists.
 *
 * @ingroup forms
 */
function manymail_lists_targets_lists_form($form, &$form_state) {
  $options = array();
  $lists = _manymail_lists_valid_lists();

  $header = array(
    'display_name' => t('Name'),
    'description' => t('Description'),
    'amount' => t('Recipients'),
  );

  foreach ($lists as $name => $list) {
    $options[$name] = array(
      'display_name' => check_plain($list->display_name),
      'description' => check_plain($list->description),
      'amount' => $list->amount,
    );
  }

  $form['allowed_lists'] = array(
    '#type' => 'fieldset',
    '#title' => t('Allowed recipient lists'),
    '#description' => t('Select which lists can be used as a recipient list.'),
    '#weight' => 1,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['allowed_lists']['manymail_lists_allowed_lists'] = array(
    '#type' => 'tableselect',
    '#default_value' => variable_get('manymail_lists_allowed_lists', array()),
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No lists available.'),
    '#js_select' => FALSE,
  );

  $form['allowed_lists']['manymail_lists_allowed_lists_override'] = array(
    '#type' => 'checkbox',
    '#title' => t('Certain users may ignore this restriction'),
    '#default_value' => variable_get('manymail_lists_allowed_lists_admin_override', 1),
    '#description' => t('When checked, users with the :permission permission can target any role regardless of what has been configured above.', array(':permission' => 'Override preset recipient lists')),
  );

  return system_settings_form($form);
}

/**
 * Form function for showing the recipient list management page.
 *
 * @ingroup forms
 */
function manymail_lists_targets_lists_manage_form($form, &$form_state) {
  $options = array();
  $lists = _manymail_lists_all_lists();

  $header = array(
    t('Name'),
    t('Description'),
    array('data' => t('Actions'), 'colspan' => 2),
  );

  foreach ($lists as $name => $list) {
    $options[] = array(
      check_plain($list->display_name),
      check_plain($list->description),
      l(t('edit'), "admin/config/manymail/targets/lists/manage/{$list->name}/edit"),
      l(t('delete'), "admin/config/manymail/targets/lists/manage/{$list->name}/delete"),
    );
  }

  $form['lists'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $options,
    '#empty' => t('No lists available.'),
  );

  return $form;
}

/**
 * Form function for creating a recipient list.
 *
 * @ingroup forms
 */
function manymail_lists_targets_lists_manage_create_form($form, &$form_state) {
  // Share validation for create and edit functions.
  $form['#validate'] = array('manymail_lists_targets_lists_manage_create_edit_form_validate');

  // Make the first $roles checkbox a select-all.
  $js_setting['ManyMailCheckAll'] = array(
    'list' => '#edit-list-roles',
    'key' => '#edit-list-roles-' . DRUPAL_AUTHENTICATED_RID,
  );
  drupal_add_js($js_setting, 'setting');
  drupal_add_js(drupal_get_path('module', 'manymail') . '/misc/checkall.js');

  $form['details'] = array(
    '#type' => 'fieldset',
    '#title' => t('List details'),
    '#weight' => 1,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['details']['display_name'] = array(
    '#type' => 'textfield',
    '#title' => t('List name'),
    '#description' => t('The name under which this list will appear in the selection table.'),
    '#size' => 32,
    '#required' => TRUE,
  );

  $form['details']['name'] = array(
    '#type' => 'machine_name',
    '#maxlength' => 32,
    '#machine_name' => array(
      'exists' => 'manymail_lists_list_exists',
      'source' => array('details', 'display_name'),
    ),
    '#description' => t('A unique machine-readable name for this list. It must only contain lowercase letters, numbers, and underscores.'),
    '#disabled' => FALSE,
  );

  $form['details']['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Description'),
    '#description' => t("Describe who's in this list. This description will show up in the list selection table."),
    '#required' => TRUE,
  );

  $form['roles'] = array(
    '#type' => 'fieldset',
    '#title' => t('List roles'),
    '#description' => t('Add certain user roles to the recipient list.'),
    '#weight' => 2,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // Select all but anonymous role.
  $roles = user_roles(TRUE);

  array_walk($roles, 'check_plain');
  $form['roles']['list_roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('List roles'),
    '#title_display' => 'invisible',
    '#options' => $roles,
  );

  $form['custom'] = array(
    '#type' => 'fieldset',
    '#title' => t('Custom recipients'),
    '#description' => t('Add non-user recipients to the list.<br />Each recipient should be on a separate line, following the pattern: mail|name (e.g.: jd@example.com|John Doe).<p><strong>Note: </strong>Any e-mail address entered here that already exists in the user records will be ignored.</p>'),
    '#weight' => 3,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['custom']['custom_recipients'] = array(
    '#type' => 'textarea',
    '#title' => t('Custom recipients'),
    '#title_display' => 'invisible',
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save list'),
    '#weight' => 5,
  );

  return $form;
}

/**
 * Implements submission from the Form API.
 *
 * @see manymail_lists_targets_lists_manage_create_form()
 */
function manymail_lists_targets_lists_manage_create_form_submit($form, &$form_state) {
  // We already know the list name is valid, so insert it.
  db_insert('manymail_lists')
    ->fields(array(
      'name' => $form_state['values']['name'],
      'display_name' => $form_state['values']['display_name'],
      'description' => $form_state['values']['description'],
    ))
    ->execute();

  // Save the role IDs to the database.
  $roles = array_filter($form_state['values']['list_roles']);
  foreach ($roles as $role) {
    db_insert('manymail_lists_roles')
    ->fields(array(
      'list' => $form_state['values']['name'],
      'rid' => $role,
    ))
    ->execute();

    // If authenticated user, the mail function of Lists
    // will include all users anyway, so we can stop the
    // loop right here.
    if ($role == DRUPAL_AUTHENTICATED_RID) {
      break;
    }
  }

  // Add all custom recipients. The _validate function has
  // already done the hard work for us, so pick up the valid
  // list from the $form_state array.
  $query = db_insert('manymail_lists_custom')
    ->fields(array('list', 'mail', 'name'));

  foreach ($form_state['manymail_lists_recipients'] as $record) {
    $query->values($record);
  }

  $query->execute();

  // Finally, redirect to the list management page.
  drupal_goto('admin/config/manymail/targets/lists/manage');
}

/**
 * Form function for editing a recipient list.
 *
 * @param array $form
 *   As passed in drupal_get_form().
 * @param array $form_state
 *   As passed in drupal_get_form().
 * @param string $list
 *   The unique machine name of the list.
 *
 * @ingroup forms
 */
function manymail_lists_targets_lists_manage_edit_form($form, &$form_state, $list) {
  // Save the list name for the submission handler.
  $form_state['manymail_list'] = $list;

  // Retrieve the creation form.
  $form = manymail_lists_targets_lists_manage_create_form($form, $form_state);

  // Retrieve the metadata for this list.
  $lists = _manymail_lists_all_lists();
  $data = $lists[$list];

  // Retrieve the selected roles for this list.
  $query = db_select('manymail_lists_roles', 'ml');
  $query->addField('ml', 'rid');
  $query->condition('ml.list', $list);
  $roles = $query->execute()->fetchAllKeyed(0, 0);

  // Retrieve the custom recipients for this list.
  $query = db_select('manymail_lists_custom', 'mc');
  $query->addField('mc', 'mail');
  $query->addField('mc', 'name');
  $query->condition('mc.list', $list);

  $custom = array();
  foreach ($query->execute() as $record) {
    $custom[] = $record->mail . '|' . $record->name;
  }

  // Set the default values.
  $form['details']['display_name']['#default_value'] = $data->display_name;
  $form['details']['name']['#default_value'] = $data->name;
  $form['details']['name']['#disabled'] = TRUE;
  $form['details']['description']['#default_value'] = $data->description;
  $form['roles']['list_roles']['#default_value'] = $roles;
  $form['custom']['custom_recipients']['#default_value'] = implode("\n", $custom);

  return $form;
}

/**
 * Implements submission from the Form API.
 *
 * @see manymail_lists_targets_lists_manage_edit_form()
 */
function manymail_lists_targets_lists_manage_edit_form_submit($form, &$form_state) {
  $list = $form_state['manymail_list'];

  // Update name and description.
  db_update('manymail_lists')
    ->fields(array(
      'display_name' => $form_state['values']['display_name'],
      'description' => $form_state['values']['description'],
    ))
    ->condition('name', $list)
    ->execute();

  // Delete all previously associated roles.
  db_delete('manymail_lists_roles')
    ->condition('list', $list)
    ->execute();

  // Save the edited role IDs to the database.
  $roles = array_filter($form_state['values']['list_roles']);
  foreach ($roles as $role) {
    db_insert('manymail_lists_roles')
    ->fields(array(
      'list' => $form_state['values']['name'],
      'rid' => $role,
    ))
    ->execute();

    // If authenticated user, the mail function of Lists
    // will include all users anyway, so we can stop the
    // loop right here.
    if ($role == DRUPAL_AUTHENTICATED_RID) {
      break;
    }
  }

  // Delete all previously associated custom recipients.
  db_delete('manymail_lists_custom')
    ->condition('list', $list)
    ->execute();

  // Re-add all custom recipients.
  $query = db_insert('manymail_lists_custom')
    ->fields(array('list', 'mail', 'name'));

  foreach ($form_state['manymail_lists_recipients'] as $record) {
    $query->values($record);
  }

  $query->execute();

  // Finally, redirect to the list management page.
  drupal_goto('admin/config/manymail/targets/lists/manage');
}

/**
 * Implements validation from the Form API.
 *
 * @see manymail_lists_targets_lists_manage_create_form()
 * @see manymail_lists_targets_lists_manage_edit_form()
 */
function manymail_lists_targets_lists_manage_create_edit_form_validate($form, &$form_state) {
  // Make line endings uniform for the explode() call.
  $recipients = preg_replace("/(\r\n|\r)+/", "\n", $form_state['values']['custom_recipients']);
  $recipients = explode("\n", $recipients);

  $valid = array();
  $invalid = array();
  $lookup = array();
  $duplicates = array();

  foreach ($recipients as $recipient) {
    // Skip entirely (without logging) if the string is empty;
    if (empty($recipient)) {
      continue;
    }

    // Skip if pipe character was not found.
    if (strpos($recipient, '|') === FALSE) {
      $invalid[] = $recipient;
      continue;
    }

    $recipient_data = explode('|', $recipient);

    // Skip if more than one pipe character was present.
    if (count($recipient_data) > 2) {
      $invalid[] = $recipient;
      continue;
    }

    // Make sure the provided e-mail address is valid.
    $mail = filter_var($recipient_data[0], FILTER_VALIDATE_EMAIL);
    $name = $recipient_data[1];

    if ($mail) {
      if (in_array($mail, $lookup)) {
        $duplicates[] = $mail;
        continue;
      }
      $lookup[] = $mail;

      $valid[] = array(
        'list' => $form_state['values']['name'],
        'mail' => check_plain($mail),
        'name' => check_plain($name),
      );
    }
    else {
      $invalid[] = $recipient;
    }
  }

  if ($invalid) {
    $bad_lines = '';
    foreach ($invalid as $line) {
      $bad_lines .= '<li>' . check_plain($line) . '</li>';
    }

    form_set_error('custom_recipients', t('The following custom recipient lines were incorrectly formatted: !lines', array('!lines' => "<ul>$bad_lines</ul>")));
  }
  elseif ($duplicates) {
    $duplicates = array_unique($duplicates);

    form_set_error('custom_recipients', t('The following e-mail addresses were entered multiple times: @duplicates.', array('@duplicates' => implode(', ', $duplicates))));
  }
  else {
    $form_state['manymail_lists_recipients'] = $valid;
  }
}

/**
 * Form function for deleting a recipient list.
 *
 * @param array $form
 *   As passed in drupal_get_form().
 * @param array $form_state
 *   As passed in drupal_get_form().
 * @param string $list
 *   The unique machine name of the list.
 *
 * @ingroup forms
 */
function manymail_lists_targets_lists_manage_delete_form($form, &$form_state, $list) {
  if (isset($form_state['values']['op']) && $form_state['values']['op'] == $form_state['values']['cancel']) {
    drupal_goto('admin/config/manymail/targets/lists/manage');
  }

  // Save the list name for the submission handler.
  $form_state['manymail_list'] = $list;

  // Retrieve the data for this list.
  $lists = _manymail_lists_all_lists();
  $data = $lists[$list];

  $form['message'] = array(
    '#type' => 'item',
    '#title' => t('Confirm deletion'),
    '#description' => t('Are you sure you want to delete the following list: <strong>@list</strong>?', array('@list' => $data->display_name)),
  );

  $form['cancel'] = array(
    '#type' => 'button',
    '#value' => t('Cancel'),
  );

  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
  );

  return $form;
}

/**
 * Implements submission from the Form API.
 *
 * @see manymail_lists_targets_lists_manage_delete_form()
 */
function manymail_lists_targets_lists_manage_delete_form_submit($form, &$form_state) {
  $list = $form_state['manymail_list'];

  db_delete('manymail_lists_custom')
    ->condition('list', $list)
    ->execute();

  db_delete('manymail_lists_roles')
    ->condition('list', $list)
    ->execute();

  db_delete('manymail_lists')
    ->condition('name', $list)
    ->execute();

  drupal_goto('admin/config/manymail/targets/lists/manage');
}
